/*
SimpleFirmata  2010/09/04
tahara
*/
//Digital is PIN 1~13 & AnalogIN(0~5) is PIN (16~20).->DigitaRead&DigitalWrite
//PWM is D3,D9,D10,D11. ->AnalogWrite
//AnalogIn is 0~5. ->AnalogRead

#include <Firmata.h>

byte previousPIN[3];  // PIN means PORT for input
byte previousPORT[3]; 

void outputPort(byte portNumber, byte portValue)
{
// only send the data when it changes, otherwise you get too many messages!
    if(previousPIN[portNumber] != portValue) {
        Firmata.sendDigitalPort(portNumber, portValue); 
        previousPIN[portNumber] = portValue;
    }
}

void setPinModeCallback(byte pin, int mode) {
    if(pin > 1) { // don't touch RxTx pins (0,1)
        pinMode(pin, mode);
    }
}

void digitalWriteCallback(byte port, int value)
{
    byte i;
    byte currentPinValue, previousPinValue;

    if(value != previousPORT[port]) {
        for(i=0; i<8; i++) {
            currentPinValue = (byte) value & (1 << i);
            previousPinValue = previousPORT[port] & (1 << i);
            if(currentPinValue != previousPinValue) {
                digitalWrite(i + (port*8), currentPinValue);
            }
        }
        previousPORT[port] = value;
    }
}

// Called whenever Arduino receives an analog msg thru Firmata
void analogWriteCallback(byte pin, int value)
{
      analogWrite(pin, value);
}

void setup()
{
    Firmata.setFirmwareVersion(0, 1);
    Firmata.attach(ANALOG_MESSAGE, analogWriteCallback); // Call this function when analog writes are received
    Firmata.attach(DIGITAL_MESSAGE,digitalWriteCallback);
    Firmata.attach(SET_PIN_MODE, setPinModeCallback);
    Firmata.begin(57600);
}

byte analogPin;

void loop()
{
    outputPort(0, PIND &~ B00000011); // pins 0-7, ignoring Rx/Tx pins (0/1)
    outputPort(1, PINB); // pins 8-13
    outputPort(2, PINC);
    while(Firmata.available()) { // Handles Firmata serial input
        Firmata.processInput();
    }
        //Analog read
        for(analogPin = 0; analogPin < TOTAL_ANALOG_PINS; analogPin++) {
        Firmata.sendAnalog(analogPin, analogRead(analogPin)); 
        }
}

